%
% Areawise Significance Test
% --------------------------
%
% Perform Areawise Significance Test. This function is called by wsp() and wco()
%
%
% USAGE: 
% [sl,kernel] =
%   arealtest(wt,dt,s0,noctave,nvoice,w0,swabs,tw,siglevel,arealsiglevel,kernel,type);
%
%
% OUTPUT arguments [sl,kernel]:
%
%   sl:      binary matrix with output of arealtest
%   kernel:  bitmap of the reproducing kernel
%
%
% INPUT arguments: 
%
% Mandatory input:
%   wt:          data matrix with power spectrum of wavelet transform (abs^2)
%
% Optional input (has to appear in the right order, set -1 to obtain default):
%   dt(=1):      sampling time
%   s0(=1):      lowest calculated scale in units of the time series
%   noctave(=5): number of octaves
%   nvoice(=10): number of voices per octave
%   w0(=2*pi):   time/frequency resolution omega_0, parameter of Morlet Wavelet
%   swabs(=0):   smoothing window width in scale direction
%   tw(=0):      length of smoothing window in time direction is 2*s*tw+1
%   siglevel:    significance level, e.g. 0.9, 0.95 or 0.99. siglevel might also be a vector, e.g. [0.9 0.95] to plot more contourlines.
%   arealsiglevel(=0.9):  significance level of the areawise test; 
%                currently only for siglevel=0.95 and for arealsiglevel=0.9 possible, 
%                i.e. 90 percent of the area of false positive patches is sorted out
%   kernel(=0):  kernel bitmap, if not provided a new kernel bitmap is calculated
%   type(=0):    
%
%
%--------------------------------------------------------------------------
%   Copyright (C) 2004-2007, Douglas Maraun and Jonas Laehnemann (Matlab adaption)
%
%   This software may be used, copied, or redistributed as long as it is not
%   sold and this copyright notice is reproduced on each copy made. This
%   routine is provided as is without any express or implied warranties
%   whatsoever.
%   
%   URL: http://tocsy.agnld.uni-potsdam.de/wavelets/
%   This package is also available as R routine.
%
% Notice: Please acknowledge the use of this software in any publications:
%    ``Wavelet coherency software was provided by D. Maraun and J. Laehnemann
%      and is available at URL: http://tocsy.agnld.uni-potsdam.de/wavelets/''.
%
% References:
%   D. Maraun, J. Kurths and M. Holschneider: Nonstationary Gaussian
%   Processes in Wavelet Domain: Synthesis, Estimation and Significance
%   Testing. Phys. Rev. E 75, 016707, 2007
%
% If you have got any questions, please feel free to contact: 
%   Douglas Maraun, PhD
%   Climatic Research Unit, University of East Anglia
%   +44 1603 59 3857
%   d.maraun(at)uea.ac.uk
%   http://www.cru.uea.ac.uk/~douglas
%--------------------------------------------------------------------------

function [sl,kernel] = arealtest(wt,dt,s0,noctave,nvoice,w0,swabs,tw,siglevel,arealsiglevel,kernel,type);


  % CHECK INPUTS
  
  if (nargin < 12), type = -1; end;
  if (nargin < 11), kernel = -1; end;  
  if (nargin < 10), arealsiglevel = -1; end;
  if (nargin < 9), siglevel = -1; end;  
  if (nargin < 8), tw = -1; end;
  if (nargin < 7), swabs = -1; end;
  if (nargin < 6), w0 = -1; end;
  if (nargin < 5), nvoice = -1; end;
  if (nargin < 4), noctave = -1; end;
  if (nargin < 3), s0 = -1; end;
  if (nargin < 2), dt = -1; end;  
  if (nargin < 1) 
      error('Must input data matrix') 
  end

  % SET DEFAULTS
  
  if (dt == -1), dt=1; end;
  if (s0 == -1), s0=1; end;
  if (noctave == -1), noctave = 5; end;
  if (nvoice == -1), nvoice = 10; end;
  if (w0 == -1), w0 = 2*pi; end;
  if (swabs == -1), swabs = 0; end;
  if (tw == -1), tw = 0; end;
  if (siglevel == -1), siglevel = 0.95; end;
  if (arealsiglevel == -1), arealsiglevel = 0.9; end;
  if (kernel == -1), kernel=0; end;
  if (type == -1), type=0; end;

  % ADJUST AND CALCULATE NECESSARY PARAMETERS

  [slp,slpMessage] = slope(w0,swabs,tw,nvoice,siglevel,arealsiglevel,type);
  
  Message_arealtest = strvcat('### Areawise Significance Test',slpMessage);
  
  % CREATE KERNEL
  
  if (~isstruct(kernel))
      
    maxarea = s0*2^noctave*slp/10*nvoice/dt;
    
    Message_arealtest = strvcat(Message_arealtest,['# calculating kernel (maxarea=' num2str(maxarea) ')']);
    
    cvkernel = kernelRoot(s0,w0,maxarea,noctave,nvoice,swabs,tw,dt);

    Message_arealtest = strvcat(Message_arealtest,'# building kernel bitmap');
    
    kernel = kernelBitmap(cvkernel,s0,w0,noctave,nvoice,swabs,tw,dt);
    
  end;

  Message_arealtest = strvcat(Message_arealtest,'# performing arealtest');
  Message_arealtest
  
  % CALCULATE AREAL SIGNIFICANCE LEVELS
  sl = slide(wt,kernel,s0,noctave,nvoice,1);
  
end


%-------------------------------------------------------------------------%


function [mark] = slide(data,kernellist,s0,noctave,nvoice,cv);

  % slides kernel over data set
  % ---------------------------
  % data:       matrix containing data
  % bitmap:     kernel bitmap
  % js:         kernel is
  % s0:         lowest scale
  % noctave:    number of octaves
  % nvoice:     number of voices per octave
  % cv:         critical value, all values higher are set to one

  %Time:  (rows) n,i the kernel is to be scaled in this direction 
  %Scale: (cols) m,j

  bitmap = kernellist.bitmap; % get kernel bitmap and is from structure kernellist
  js = kernellist.is;
  
  data(data<cv) = 0; % set all values to zero that are not above the critical value
  dbin = tobin(data); % create binary matrix of pointwise critical values
  
  kbin = tobin(bitmap); % kenrel bitmap to binary
  
  sm = s0*2^noctave; % maximum scale

  [dn,dm] = size(dbin); % dimensions of data matrix

  [kn,km] = size(kbin); % dimensions of matrix containing kernel bitmap

  mark = zeros(dn,dm); % initialise matrix for test results

  for (j = 1:(dm-km+1)) % slide image in m-direction
   
    s = s0*2^((j+js-1)/nvoice);
    kscn = fix(kn*s/sm);
    
    if (kscn == 0) kscn = 1; end;

    ksc = scaleKernel(kbin,kscn);
    kscm = km;
    
    for (i = 1:(dn-kscn+1)) % slide image in n-direction
      
      subbin = dbin(i:(i+kscn-1),j:(j+kscm-1));
      if (sum(sum(ksc.*subbin)) == sum(sum(ksc)))
        mark(i:(i+kscn-1),j:(j+kscm-1)) = mark(i:(i+kscn-1),j:(j+kscm-1))+ksc;
      end;
      
    end;

  end;

  mark = tobin(mark); % convert to binary matrix
  
end


%-------------------------------------------------------------------------%


function [kernel] = kernelBitmap(c,s0,w0,noctave,nvoice,swabs,tw,dt);

  % produces kernel bitmap
  % ----------------------
  % c:       height of contour, that defines area
  % s0:      lowest scale
  % noctave: number of octaves
  % nvoice:  number of voices per octave
  % swabs:   smoothing window length in scale direction
  % dt:      sampling time

  s = s0*2^noctave;
  is = noctave*nvoice;

  x = s0.*2.^(((1:(nvoice*(noctave+2)))-1)./nvoice);

  t = max(2000,max(x)*tw*2/dt*1.1);

  y = ((0:fix(2*t))-t)./2.*dt;

  X = repmat(x,fix(2*t+1),1);
  Y = repmat(y',1,nvoice*(noctave+2));

  F = sqrt(2*s.*X./(s*s+X.*X)).*exp(-0.5.*(Y.*Y+w0*w0.*(X-s).*(X-s))./(s*s+X.*X));
  
  F = foldKernel(F,swabs,tw,x,dt);

  F(F<c) = 0;
  F(F>=c) = 1;

  is1 = 1;
  is2 = nvoice*(noctave+1);
  it1 = 1;
  it2 = fix(2*t+1);

  L = F(1:fix(2*t+1),is1);
  
  
  while (length(L(L~=0)) == 0)
    is1 = is1+1;
    L = F(1:fix(2*t+1),is1);
  end;
  
  L = F(1:fix(2*t+1),is2);

  while (length(L(L~=0)) == 0)
    is2 = is2-1;
    L = F(1:fix(2*t+1),is2);
  end;
  
  L = F(it1,1:(nvoice*(noctave+2)));

  while (length(L(L~=0)) == 0)
    it1 = it1+1;
    L = F(it1,1:(nvoice*(noctave+2)));
  end;

  L = F(it2,1:(nvoice*(noctave+2)));

  while (length(L(L~=0)) == 0)
    it2 = it2-1;
    L = F(it2,1:(nvoice*(noctave+2)));
  end;

  kernel.bitmap = F((it1-1):(it2+1),(is1-1):(is2+1));
  kernel.is=is-is1;

end


%-------------------------------------------------------------------------%


function [cntr] = kernelRoot(s0,w0,a,noctave,nvoice,swabs,tw,dt);

  % calculates height of contour line to define kernel area
  % -------------------------
  % s0:      lowest scale
  % w0:      parameter of Morlet Wavelet
  % a:       area offset
  % noctave: number of octaves
  % nvoice:  number of voices per octave
  % swabs:   smoothing window length in scale direction
  % dt:      sampling time

  tol = 0.005;
  cmin = 0;
  cmax = 1;
  cntr = 0.5;
  da = a;

  while abs(da/a) > tol

    da = kernelArea(cntr,s0,w0,a,noctave,nvoice,swabs,tw,dt);
    
    if (da>0)
      cmin = cntr;
      cntr = mean([cntr,cmax]);
    end;

    if (da<0)
      cmax = cntr;
      cntr = mean([cntr,cmin]);   
    end;

  end;
  
end


%-------------------------------------------------------------------------%


function [area] = kernelArea(cntr,s0,w0,a,noctave,nvoice,swabs,tw,dt);

  % calulates area of reproducing kernel for smoothed data at scale s0*2^noctave
  % ------------------------------------
  % cntr:    height of contour line to define kernel area. This
  %          parameter is to be estimated!
  % s0:      lowest scale
  % w0:      parameter of Morlet Wavelet
  % a:       area offset (only needed, when finding root. Is set to
  %          desired area)
  % noctave: number of octaves
  % nvoice:  number of voices per octave
  % swabs:   smoothing window width in scale direction
  % dt:      sampling time
 
  s = s0*2^noctave; % maximum scale
  
  x = s0.*2.^(((1:(nvoice*(noctave+2)))-1)./nvoice);

  t = max(2000,max(x)*tw*2/dt*1.1);
  
  y = ((0:(2*t))-t)./2.*dt;
  
  X = repmat(x,fix(2*t+1),1);
  Y = repmat(y',1,nvoice*(noctave+2));

  F = sqrt(2*s.*X./(s*s+X.*X)).*exp(-0.5.*(Y.*Y+w0*w0.*(X-s).*(X-s))./(s*s+X.*X));

  F = foldKernel(F,swabs,tw,x,dt);
  
  F(F>=cntr) = 1;
  F(F<cntr) = 0;
  
  area = length(F(F==1))-a;

end


%-------------------------------------------------------------------------%

  
function [scF] = foldKernel(F,swabs,tw,s,dt);

  % folds a matrix (e.g. kernel with smoothing window)
  % --------------
  % F:  matrix input
  % swabs: smoothing window width

  smsF = smoothmatrix(F,swabs);
  smsF(isnan(smsF)) = 0;
  
  smtsF = smoothtime(smsF,tw,dt,s);

  smtsF(isnan(smtsF)) = 0;
    
  scF = scalematrix(smtsF);
  
end


%-------------------------------------------------------------------------%


function [newKernel] = scaleKernel(kernel,l);

  % scales kernel length in time direction proportional to scale
  % --------------------
  % kernel: data bitmap of width n
  % l:      new width of kernel
  
  [n,m] = size(kernel); % dimensions of kernel

  newKernel = zeros(l,m); % initalise kernel at new width
  
  d = n/l; % scaling factor

  for (i = 1:l) % row-wise filling of new kernel
    j = fix((i-0.5)*d);
    
    if (j == 0)
        j = 1;
    end;
    
    newKernel(i,1:m) = kernel(j,1:m);
    
  end;

end



%-------------------------------------------------------------------------%


function [x] = tobin(x);

  % sets nonzero values to one
  % --------------------------
  
  x(x~=0)=1;
  
end


%-------------------------------------------------------------------------%


function [slp,Message] = slope(w0,swabs,tw,nvoice,siglevel,arealsiglevel,type);

  %
  % --------------------
  %

  sw = swabs/nvoice;
  
  if (type==0) % wavelet spectrum

    if (tw == 0 	 & sw == 0 	 & w0 == 1 *pi) slp =  5.82518; end; 	 % w = 18.35831 
    if (tw == 1.5 	 & sw == 0 	 & w0 == 1 *pi) slp =  24.69852; end; 	 % w = 14.30709 
    if (tw == 3 	 & sw == 0 	 & w0 == 1 *pi) slp =  35.48368; end; 	 % w = 14.72354 
    if (tw == 0 	 & sw == 5 	 & w0 == 1 *pi) slp =  7.347707; end; 	 % w = 17.96942 
    if (tw == 1.5 	 & sw == 5 	 & w0 == 1 *pi) slp =  28.24291; end; 	 % w = 12.65993 
    if (tw == 3 	 & sw == 5 	 & w0 == 1 *pi) slp =  51.13723; end; 	 % w = 10.96359 
    if (tw == 0 	 & sw == 10 	 & w0 == 1 *pi) slp =  10.47856; end; 	 % w = 15.5941 
    if (tw == 1.5 	 & sw == 10 	 & w0 == 1 *pi) slp =  45.07387; end; 	 % w = 15.29793 
    if (tw == 3 	 & sw == 10 	 & w0 == 1 *pi) slp =  52.82886; end; 	 % w = 12.72361 

    if (tw == 0 	 & sw == 0 	 & w0 == 2 *pi) slp =  8.718912; end; 	 % w = 17.75933 
    if (tw == 1.5 	 & sw == 0 	 & w0 == 2 *pi) slp =  11.88006; end; 	 % w = 15.39648 
    if (tw == 3 	 & sw == 0 	 & w0 == 2 *pi) slp =  26.55977; end; 	 % w = 1.064384 
    if (tw == 0 	 & sw == 5 	 & w0 == 2 *pi) slp =  14.64761; end; 	 % w = 16.27518 
    if (tw == 1.5 	 & sw == 5 	 & w0 == 2 *pi) slp =  28.27798; end; 	 % w = 14.57059 
    if (tw == 3 	 & sw == 5 	 & w0 == 2 *pi) slp =  63.54121; end; 	 % w = 12.83778 
    if (tw == 0 	 & sw == 10 	 & w0 == 2 *pi) slp =  27.78735; end; 	 % w = 11.95813 
    if (tw == 1.5 	 & sw == 10 	 & w0 == 2 *pi) slp =  41.27260; end; 	 % w = 12.03379 
    if (tw == 3 	 & sw == 10 	 & w0 == 2 *pi) slp =  67.37015; end; 	 % w = 10.63935 

  end;

  if (type==1) % wavelet coherence

    if (tw==0   & sw==0   & w0==pi) slp = 999; end; %not valid
    if (tw==1.5 & sw==0   & w0==pi) slp = 1; end;
    if (tw==3   & sw==0   & w0==pi) slp = 1; end;
    if (tw==0   & sw==0.5 & w0==pi) slp = 1; end;
    if (tw==1.5 & sw==0.5 & w0==pi) slp = 1; end;
    if (tw==3   & sw==0.5 & w0==pi) slp = 1; end;
    if (tw==0   & sw==1   & w0==pi) slp = 1; end;
    if (tw==1.5 & sw==1   & w0==pi) slp = 1; end;
    if (tw==3   & sw==1   & w0==pi) slp = 1; end;

    if (tw==0   & sw==0   & w0==2*pi) slp = 999; end; %not valid
    if (tw==1.5 & sw==0   & w0==2*pi) slp = 1; end;
    if (tw==3   & sw==0   & w0==2*pi) slp = 1; end;
    if (tw==0   & sw==0.5 & w0==2*pi) slp = 1; end;
    if (tw==1.5 & sw==0.5 & w0==2*pi) slp = 8.3; end;
    if (tw==3   & sw==0.5 & w0==2*pi) slp = 1; end;
    if (tw==0   & sw==1   & w0==2*pi) slp = 1; end;
    if (tw==1.5 & sw==1   & w0==2*pi) slp = 1; end;
    if (tw==3   & sw==1   & w0==2*pi) slp = 1; end;

    if (tw==0   & sw==0   & w0==3*pi) slp = 999; end; %not valid
    if (tw==1.5 & sw==0   & w0==3*pi) slp = 1; end;
    if (tw==3   & sw==0   & w0==3*pi) slp = 1; end;
    if (tw==0   & sw==0.5 & w0==3*pi) slp = 1; end;
    if (tw==1.5 & sw==0.5 & w0==3*pi) slp = 1; end;
    if (tw==3   & sw==0.5 & w0==3*pi) slp = 1; end;
    if (tw==0   & sw==1   & w0==3*pi) slp = 1; end;
    if (tw==1.5 & sw==1   & w0==3*pi) slp = 1; end;
    if (tw==3   & sw==1   & w0==3*pi) slp = 1; end;

    if (tw==0   & sw==0   & w0==4*pi) slp = 999; end; %not valid
    if (tw==1.5 & sw==0   & w0==4*pi) slp = 1; end;
    if (tw==3   & sw==0   & w0==4*pi) slp = 1; end;
    if (tw==0   & sw==0.5 & w0==4*pi) slp = 1; end;
    if (tw==1.5 & sw==0.5 & w0==4*pi) slp = 1; end;
    if (tw==3   & sw==0.5 & w0==4*pi) slp = 1; end;
    if (tw==0   & sw==1   & w0==4*pi) slp = 1; end;
    if (tw==1.5 & sw==1   & w0==4*pi) slp = 1; end;
    if (tw==3   & sw==1   & w0==4*pi) slp = 1; end;

  end;

  Message = ['# slope ' num2str(slp)];
  
end


