%
% Pointwise Significance Test - WCO
% ---------------------------------
%
% This function estimates critical values for the wavelet coherency between
% two processes using the Morlet wavelet based on a formula derived from
% Monte Carlo simulations. 
%
%
% USAGE
%   cv = criticalvaluesWCO(s0,noctave,nvoice,w0,swabs,tw,siglevel);
%
%
% OUTPUT value
%
%   cv:    scale independent critical value. If siglevel is a vector of
%          multiple significance values, also the return value is a vector
%          of the same length.  
%
%
% INPUT arguments (all optional):
%
%   s0(=1):          lowest calculated scale in units of the time series
%   noctave(=5):     number of octaves
%   nvoice(=20):     number of voices per octave
%   w0(=2*pi):       time/frequency resolution omega_0, parameter of Morlet Wavelet
%   swabs(=0):       length of smoothing window in scale direction at scale s is 2*swabs+1
%   tw(=0):          length of smoothing window in time direction is 2*s*tw+1
%   siglevel(=0.95): significance level, e.g. 0.9, 0.95 or 0.99. At the
%                    moment, only these values are possible. siglevel might
%                    also be a vector, e.g. c(0.9,0.95) to plot more contourlines.  
%
%
% DETAILS
% The process dependency appeared to be rather marginal. Thus we performed
% MC simulations with two Gaussian White Noise processes for the listed
% significance levels for different smoothing windows and time/frequency
% resolutions.
%
%
%--------------------------------------------------------------------------
%   Copyright (C) 2004-2007, Douglas Maraun and Jonas Laehnemann (Matlab adaption)
%
%   This software may be used, copied, or redistributed as long as it is not
%   sold and this copyright notice is reproduced on each copy made. This
%   routine is provided as is without any express or implied warranties
%   whatsoever.
%   
%   URL: http://tocsy.agnld.uni-potsdam.de/wavelets/
%   This package is also available as R routine.
%
% Notice: Please acknowledge the use of this software in any publications:
%    ``Wavelet coherency software was provided by D. Maraun and J. Laehnemann
%      and is available at URL: http://tocsy.agnld.uni-potsdam.de/wavelets/''.
%
% References:
%   D. Maraun, J. Kurths and M. Holschneider: Nonstationary Gaussian
%   Processes in Wavelet Domain: Synthesis, Estimation and Significance
%   Testing. Phys. Rev. E 75, 016707, 2007
%
%   D. Maraun and J. Kurths: Cross Wavelet Analysis. Significance Testing
%   and Pitfalls, Nonlin. Proc. Geoph. 11(4), 505-514, 2004
%
% If you have got any questions, please feel free to contact: 
%   Douglas Maraun, PhD
%   Climatic Research Unit, University of East Anglia
%   +44 1603 59 3857
%   d.maraun(at)uea.ac.uk
%   http://www.cru.uea.ac.uk/~douglas
%--------------------------------------------------------------------------


function [cv] = criticalvaluesWCO(s0,noctave,nvoice,w0,swabs,tw,siglevel);

  cv=zeros(length(siglevel),1);
  Message_criticalvaluesWCO = '### Pointwise Significance Test';
  
  for (i = 1:length(siglevel))

    if (siglevel(i)~=0.9 & siglevel(i)~=0.95 & siglevel(i)~=0.99) 
        siglevel(i) = 0.95;
    end;

    if (siglevel(i)==0.9)
      Message_criticalvaluesWCO = strvcat(Message_criticalvaluesWCO,'# significance level set to 0.90');
      sw = 1.0*swabs/nvoice;
      cv(i) = 0.00246872*w0^2*sw + 0.0302419*w0*sw^2 + 0.342056*sw^3 - 0.000425853*w0^2 - 0.101749*w0*sw - 0.703537*sw^2 + 0.00816219*w0 + 0.442342*sw + 0.970315;
    end;
  
    if (siglevel(i)==0.95)
      Message_criticalvaluesWCO = strvcat(Message_criticalvaluesWCO,'# significance level set to 0.95');
      sw = swabs*100.0/3.0/nvoice;
      cv(i) = 0.0000823*w0^3 + 0.0000424*w0^2*sw + 0.0000113*w0*sw^2 + 0.0000154*sw^3 - 0.0023*w0^2 - 0.00219*w0*sw - 0.000751*sw^2 + 0.0205*w0 + 0.0127*sw + 0.95;
    end;
    
    if (siglevel(i)==0.99)
      Message_criticalvaluesWCO = strvcat(Message_criticalvaluesWCO,'# significance level set to 0.99');
      sw = 1.0*swabs/nvoice;
      cv(i) = 0.00102304*w0^2*sw +  0.00745772*w0*sw^2 + 0.230035*sw^3 - 0.000361565*w0^2 - 0.0502861*w0*sw - 0.440777*sw^2 + 0.00694998*w0 + 0.29643*sw + 0.972964;
    end;

    if (cv(i)>1) 
        cv(i) = 1; 
    end;
  
    Message_criticalvaluesWCO = strvcat(Message_criticalvaluesWCO,['# significance testing, cv=' num2str(cv(i))]);
    
  end;
  
  Message_criticalvaluesWCO

end