%
% Pointwise Significance Test - WSP
% ---------------------------------
%
% This function estimates critical values for the Wavelet spectra by means
% of Monte Carlo simulations. Null Hypothesis is an AR1 (red noise) process
% fitted to the given time series.
%
%
% NEEDS: 
%   - Matlab 'System Identification Toolbox' for ar and sim
%   - Wavelet software by C. Torrence and G. Compo
%       available at URL: http://paos.colorado.edu/research/wavelets/
%
%
% USAGE
%   cv = criticalvaluesWSP(ts,s0,noctave,nvoice,w0,swabs,tw,siglevel,nreal);
%
%
% OUTPUT value
%
%   cv: matrix of scale DEPENDENT critical values. The number of rows of
%       the matrix corresponds to the number of chosen significance values. The
%       number of columns equals the number of scales.
%
%
% INPUT arguments and possible default values
%
% Mandatory input:
%   ts            time series 
%
% Optional input (has to appear in the right order, set -1 to obtain default):
%   s0(=1)          lowest calculated scale in units of the time series
%   noctave(=5) 	number of octaves
%   nvoice(=10) 	number of voices per octave
%   w0(=2*pi)       time/frequency resolution omega_0
%   swabs(=0)       length of smoothing window is 2swabs+1
%   tw(=0)          length of smoothing window in time direction is 2*s*tw+1
%   siglevel(=0.95) significance level, e.g. 0.9, 0.95 or 0.99. siglevel might also be a vector, e.g. [0.9 0.95] to plot more contourlines.
%   nreal(=1000) 	number of realizations to estimate critical values for the corresponding significance values, default 1000
%
%
% DETAILS:
% nreal might be chosen as 100 for a rough estimate of significance.
% However, it is for sure not suitable to reliably distinguish between 95
% and 99 percent significance values. In this case, at least nreal=1000
% should be chosen.
%
%
%--------------------------------------------------------------------------
%   Copyright (C) 2004-2007, Douglas Maraun and Jonas Laehnemann (Matlab adaption)
%
%   This software may be used, copied, or redistributed as long as it is not
%   sold and this copyright notice is reproduced on each copy made. This
%   routine is provided as is without any express or implied warranties
%   whatsoever.
%   
%   URL: http://tocsy.agnld.uni-potsdam.de/wavelets/
%   This package is also available as R routine.
%
% Notice: Please acknowledge the use of this software in any publications:
%    ``Wavelet coherency software was provided by D. Maraun and J. Laehnemann
%      and is available at URL: http://tocsy.agnld.uni-potsdam.de/wavelets/''.
%
% References:
%   D. Maraun, J. Kurths and M. Holschneider: Nonstationary Gaussian
%   Processes in Wavelet Domain: Synthesis, Estimation and Significance
%   Testing. Phys. Rev. E 75, 016707, 2007
%
%   D. Maraun and J. Kurths: Cross Wavelet Analysis. Significance Testing
%   and Pitfalls, Nonlin. Proc. Geoph. 11(4), 505-514, 2004
%
% If you have got any questions, please feel free to contact: 
%   Douglas Maraun, PhD
%   Climatic Research Unit, University of East Anglia
%   +44 1603 59 3857
%   d.maraun(at)uea.ac.uk
%   http://www.cru.uea.ac.uk/~douglas
%--------------------------------------------------------------------------



function [cv] = criticalvaluesWSP(ts,s0,noctave,nvoice,w0,swabs,tw,siglevel,nreal);

  % CHECK INPUTS
  
  if (nargin < 9), nreal = -1; end;
  if (nargin < 8), siglevel = -1; end;  
  if (nargin < 7), tw = -1; end;
  if (nargin < 6), swabs = -1; end;
  if (nargin < 5), w0 = -1; end;
  if (nargin < 4), nvoice = -1; end;
  if (nargin < 3), noctave = -1; end;
  if (nargin < 2), s0 = -1; end;
  if (nargin < 1) 
      error('Must input an iddata TS object') 
  end

  % SET DEFAULTS
  
  if (s0 == -1), s0=1; end;
  if (noctave == -1), noctave = 5; end;
  if (nvoice == -1), nvoice = 10; end;
  if (w0 == -1), w0 = 2*pi; end;
  if (swabs == -1), swabs = 0; end;
  if (tw == -1), tw = 0; end;
  if (siglevel == -1), siglevel = 0.95; end;
  if (nreal == -1), nreal = 1000; end;

  % CHECK TIME SERIES INPUT 

  if (size(ts,2)==2)
    y = ts(:,2)';
    t = ts(:,1)';
    dt= (ts(length(y),1)-ts(1,1))/(length(y)-1);
  end;  
  if (size(ts,2)==1)
    y = ts;
    dt = 1;
    t = (1:dt:length(y));
  end;
  if (size(ts,2)>2)
    error('please provide two column matrix for time series');end;

  % ADJUST AND CALCULATE NECESSARY PARAMETERS

  s0 = adjusts0(s0,dt);
  noctave = adjustnoctave(length(y),dt,s0,tw,noctave);

  scalevector = 2.^((0:(noctave*nvoice))/nvoice)*s0;

  Message_criticalvaluesWSP = ('# calculating critical values by means of MC-simulations')

  s0unit=s0/dt*w0/(2*pi);               
  s0log=fix((log2(s0unit)-1)*nvoice+1.5);
  
  wsp0 = rawWSP(y,dt,s0,noctave,nvoice,w0,swabs,tw,scalevector);
  
  S = size(wsp0,1);

  n1 = 1 + 2*fix(sqrt(2) * 2^((S-swabs+s0log-1)/nvoice+1));
  if (abs(n1)>32767) n1= NaN; end;
  n2 = max(scalevector)*tw*2/dt*1.1;
  n3 = 2*tw*s0*2^noctave/dt+100;
  n = max([n1,n2,n3]);
  
  center = round((n-1)/2);
  cv = zeros(length(siglevel),S);
  rmatrix = zeros(S,nreal);
  
  % Fitting AR1-process (red noise) to data using Yule-Walker approach
  [arts0,varts0] = aryule(y,1);

  % MC Simulations
  for (i = 1:nreal)

    ars = arsim(-arts0(2),varts0,randn(n+101,1)); % simulating AR1-process with the fitted model from above using zero initial state
    yars = ars(102:(n+101)); % entering the AR1-Simulation into the MC timeseries, burn-in of 101 points is left out
    
    rawwsp = rawWSP(yars,dt,s0,noctave,nvoice,w0,swabs,tw,scalevector); % wavelet transform of MC timeseries

    rmatrix(:,i) = rawwsp(:,center); % enter this MC simulation into matrix of the simulations
        
  end;

  for (s = (1+swabs):(S-swabs)) 
      rmatrix(s,:) = sort(rmatrix(s,:));
  end;

  for (i = 1:length(siglevel))
    sigindex = fix(nreal*siglevel(i));
    cvv = rmatrix(:,sigindex);
    cvv(isnan(cvv)) = 0;
    cv(i,:) = cvv;
  end;

end

%-------------------------------------------------------------------------%

function [ary] = arsim(a,var,noise);
 
  % simulates AR1 process with parameter a using noise.
  
  noise=noise*sqrt(var);

  ary = zeros(1,length(noise));
  ary(1)=noise(1);

  for (i = 2:length(noise))
    ary(i)=a*ary(i-1)+noise(i);
  end;

end


function [smsmwsp] = rawWSP(y,dt,s0,noctave,nvoice,w0,swabs,tw,scalevector);
  
  %  performs wavelet transform
  % ---------------------------
  %  raw version of wsp to estimate spectrum (for the AR simulations) 
  %  without performing significance tests - input 
  
  yanom = y-mean(y); % substract mean from timeseries

  dj=1/nvoice; % the spacing between discrete scales
  J1=nvoice*noctave; % the # of scales minus one
  
  tscwt = wavelet(yanom,dt,1,dj,s0,J1,'Morlet',w0); % performs wavelet transform and transposes
        % zeropadding is activated 

  wsp = abs(tscwt).^2; % calculates wsp

  smwsp = smoothmatrix(wsp,swabs); % smoothing in scale direction
  smsmwsp = smoothtime(smwsp,tw,dt,scalevector); % smoothing in time direction

end

