%
% PLOTWT
% -----------------------------
%
% This function plots wavelet transform objects from the functions
% wsp,wcs,wco. Modulus and (if possible) phase are plotted.
%
%
% INPUT arguments and defaults: 
%
% mandatory input:
%   wclist:             object containing the following parameters          
%   wclist.modulus      wavelet sample (cross) spectrum/coherence of dimension [length(ts)]x[nvoice*noctave+1]
%   wclist.phase        in case of cross analysis: phase
%   wclist.time         vector of times of length(intersection of ts1 and ts2)
%   wclist.s0           lowest calculated scale in units of the time series
%   wclist.noctave      number of octaves
%   wclist.nvoice       number of voices per octave
%   wclist.w0           time/frequency resolution omega_0, parameter of Morlet Wavelet
%   wclist.scales       vector of scales of length nvoice*noctave+1
%   wclist.critval      matrix of critical values
%   wclist.at           time/scale matrix of areawise significant patches
%   wclist.kernel       bitmap of the reproducing kernel
%   wclist.coi          cone of influence
%
% optional input (has to appear in the right order, set -1 to obtain default):
%   markt(=-999):     vector of times to be marked by vertical dotted lines; when set to -999 (default), no lines are plotted.
%   marks(=-999):     vector of scales to be marked by horizontal dotted lines; when set to -999 (default), no lines are plotted.
%   logscale(=false): when true, the contours are plotted in logarithmic scale
%   phase(=false):    true if phase is to be plotted
%   units(=''):       character string giving units of the data sets.
%   wctitle(='Wavelet Plot'):   title for the plot
%   split(=false):    when true, modulus and phase are splitted into two graphs. Default: false
%   color(=true):     true (default): color plot, false: gray scale
%   labsc(=1):        scale of labels, default: 1 (10pt), for manuscripts: 2 (13pt), for manuscripts or presentations: 3 (16pt), 
%                     larger font-sizes for presentations: 4 (20pt), 5 (24pt)
%   labtext(=''):     puts a label in upper left corner of the plot
%   sigplot(=3):      0: no significance test plotted, 1: results from pointwise test, 2: results from areawise test, 3: results from both tests
%
%
% DETAILS
% When cv=0 or -1 is chosen, no critical values are plotted. The cone of
% influence is marked by black lines. If cv is a vector, the routine
% assumes that every value represents a critical value, which is constant
% in scale. If cv is a matrix, it assumes every row to contain scale
% dependent critical values, each row stands for one significance level.
% Values outside the cone of influence should be interpreted very
% carefully, as they result from a significant contribution of zero padding
% at the beginning and the end of the time series. For a better
% visualization, additional dotted lines marking distinct times or scales
% might be plotted by providing the vectors markt and marks.
%
%
%--------------------------------------------------------------------------
%   Copyright (C) 2004-2007, Douglas Maraun and Jonas Laehnemann (Matlab adaption)
%
%   This software may be used, copied, or redistributed as long as it is not
%   sold and this copyright notice is reproduced on each copy made. This
%   routine is provided as is without any express or implied warranties
%   whatsoever.
%   
%   URL: http://tocsy.agnld.uni-potsdam.de/wavelets/
%   This package is also available as R routine.
%
% Notice: Please acknowledge the use of this software in any publications:
%    ``Wavelet coherency software was provided by D. Maraun and J. Laehnemann
%      and is available at URL: http://tocsy.agnld.uni-potsdam.de/wavelets/''.
%
% References:
%   D. Maraun, J. Kurths and M. Holschneider: Nonstationary Gaussian
%   Processes in Wavelet Domain: Synthesis, Estimation and Significance
%   Testing. Phys. Rev. E 75, 016707, 2007
%
%   D. Maraun and J. Kurths: Cross Wavelet Analysis. Significance Testing
%   and Pitfalls, Nonlin. Proc. Geoph. 11(4), 505-514, 2004
%
% If you have got any questions, please feel free to contact: 
%   Douglas Maraun, PhD
%   Climatic Research Unit, University of East Anglia
%   +44 1603 59 3857
%   d.maraun(at)uea.ac.uk
%   http://www.cru.uea.ac.uk/~douglas
%--------------------------------------------------------------------------


function plotwt(wclist,markt,marks,logscale,phase,units,wctitle,split,color,labsc,labtext,sigplot)

    % CHECK INPUTS
    
    if (nargin < 12), sigplot = -1; end;
    if (nargin < 11), labtext = -1; end;
    if (nargin < 10), labsc = -1; end;
    if (nargin < 9), color = -1; end;
    if (nargin < 8), split = -1; end;
    if (nargin < 7), wctitle = -1; end;
    if (nargin < 6), units = -1; end;
    if (nargin < 5), phase = -1; end;
    if (nargin < 4), logscale = -1; end;
    if (nargin < 3), marks = -1; end;
    if (nargin < 2), markt = -1; end;
    if (nargin < 1) 
        error('Must input a wavelet object') 
    end 
    
    % SET DEFAULTS
  
    if (markt == -1), markt = -999; end; 
    if (marks == -1), marks = -999; end; 
    if (logscale == -1), logscale = false; end; %!!
    if (phase == -1 | isnan(wclist.phase)), phase = false; end;
    if (units == -1 | ~ischar(units)), units = ''; end;
    if (wctitle == -1 | ~ischar(wctitle)), wctitle = 'Wavelet Plot'; end;
    if (split == -1), split = false; end;
    if (color == -1), color = true; end;
    if (labsc == -1), labsc = 1; end;
    if (labtext == -1 | ~ischar(labtext)), labtext = ''; end;
    if (sigplot == -1), sigplot = 3; end;
 
    % ADJUST AND CALCULATE NECESSARY PARAMETERS
    
    if (phase & ~split) sp1 = subplot('position',[0.1 0.6 0.8 0.32]); end; 
        % if phase is to be plotted in the same graph, position of first graph needs to be given
    
    Yticks = 2.^(fix(log2(min(wclist.scales))):fix(log2(max(wclist.scales)))); 
        % calculate vector for the scale labels
    
    switch labsc % check labsc and set FontSize
        case 5
            fsize=24;
        case 4
            fsize=20;
        case 3 
            fsize=16;
        case 2 
            fsize=13;
        otherwise 
            fsize=10;
    end;
    
    if isempty(units) % set axes labels
        xlab = 'Time';
        ylab = 'Scale';
    else
        xlab = ['Time [' units ']'];
        ylab = ['Scale ['  units ']'];
    end;
    
    
    % PLOT MODULUS
    logscale

    if (logscale) % plot, z-direction in logarithmic or normal scale
        if (wclist.modulus(1,1)==0) wclist.modulus(1,1)=1; end;
        imagesc(wclist.time,log2(wclist.scales),log10(wclist.modulus')); % image plot logarithmic modulus
        
        tickv = ((0:5).*(min(min(log10(wclist.modulus))))./5)'; % colorbar labels
        labelv = flipud(num2str(((0:5).*(-min(min(log10(wclist.modulus))))./5)',2));
    else
        imagesc(wclist.time,log2(wclist.scales),wclist.modulus'); % image plot normal modulus
        
        tickv = (0:5).*(max(max(wclist.modulus))-min(min(wclist.modulus)))./5; % colorbar labels
        labelv = num2str([0;0.2;0.4;0.6;0.8;1],2);
    end;  
      
    if (color)
        wtcolors=hsv(91); % create colormap
        wtcolors=flipud(wtcolors(1:64,:));
    else wtcolors=gray(91); wtcolors = wtcolors(28:91,:); end; 
        % create grey colormap
        
    colormap(wtcolors)
    
    % adjust axes and title
    xlabel(xlab,'FontSize',fsize)
    ylabel(ylab,'FontSize',fsize)
    title(wctitle,'FontWeight','bold','FontSize',fsize)
    set(gca,'YLim',log2([min(wclist.scales),max(wclist.scales)]),'YDir','normal','YTick',log2(Yticks(:)),'YTickLabel',Yticks,'FontSize',fsize) %Y-ticks
    text(wclist.time(fix(0.03*length(wclist.time))),fix(log2(wclist.scales(length(wclist.scales)-1))),labtext,'FontSize',fsize) %add optional text
    
    plotcoi(wclist.time,wclist.coi); 
        % call cone of influence plot
    
    plotcv(wclist.time,wclist.scales,wclist.critval,wclist.modulus,sigplot); 
        % call plotting of pointwise significance test
    
    plotat(wclist.time,wclist.scales,wclist.at,sigplot); 
        % call plotting of areawise significance test
    
    plotmarks(wclist.time,wclist.scales,marks,markt); 
        % call plotting of marking lines
    
    %colorbar('FontSize',fsize)
    colorbar('YTick',tickv,'YTickLabel',labelv,'FontSize',fsize) 
        % add colorbar to the right

    hold off % end first plot
    
    
    % PLOT PHASE
    
    if (phase)
        
        if (color) wt2colors = hsv(64); % create colormap for 2nd plot
        else dummy = gray(64); wt2colors(1:32,:) = dummy(33:64,:); wt2colors(33:64,:) = dummy(1:32,:); end; 
            % create gray colormap for 2nd plot
        
        if (~split) % checks if phase comes into same figure or new one
            sp2 = subplot('position',[0.1 0.1 0.8 0.32]);             
        else 
            figure();
        end;
                
        imagesc(wclist.time,log2(wclist.scales),wclist.phase'); % image plot
            % plot the phase
  
        if (split) % set colormap
            colormap(wt2colors);
        else % procedure to have two different colormaps in one plot
            colormap([wtcolors;wt2colors]);
            clim1      = get(sp1,'CLim');% CLim values for each axis
            clim2      = get(sp2,'CLim');
            set(sp1,'CLim',newclim(1,64,clim1(1),clim1(2),128));
            set(sp2,'CLim',newclim(65,128,clim2(1),clim2(2),128));
        end;
            
        % adjust axes and title
        title('Phase','FontWeight','bold','FontSize',fsize)
        xlabel(xlab,'FontSize',fsize)
        ylabel(ylab,'FontSize',fsize)
        set(gca,'YLim',log2([min(wclist.scales),max(wclist.scales)]),'YDir','normal','YTick',log2(Yticks(:)),'YTickLabel',Yticks,'FontSize',fsize)
        
        plotcoi(wclist.time,wclist.coi);
            % call cone of influence plot
        
        plotcv(wclist.time,wclist.scales,wclist.critval,wclist.modulus,sigplot); 
            % call plotting of pointwise significance test
        
        plotat(wclist.time,wclist.scales,wclist.at,sigplot);
            % call plotting of areawise significance test
    
        plotmarks(wclist.time,wclist.scales,marks,markt);
            % call plotting of marking lines
            
        colorbar('YTick',[-pi -pi/2 0 pi/2 pi],'YTickLabel',{'-pi' '-pi/2' '0' 'pi/2' 'pi'},'YLim',[-pi pi],'FontSize',fsize)
            % add colorbar to the right
        
        hold off % end second plot
        
    end;

end


%-------------------------------------------------------------------------%


% plot pointwise significance test
function plotcv(time,scales,critval,modulus,sigplot);
    if (isnan(critval)==false & critval(1)~=0 & critval(1)~=-1 & (sigplot==1 | sigplot==3)) 
            % check if this plot is desired and proper values given
        hold on
        if (size(critval)==[1 1]) % single critval
            contour(time,log2(scales),modulus',[critval critval],'k');
        else
          sig = (critval')*(ones(1,length(time)));  % expand critval --> (J+1)x(N) array, if critval is vector
          sig = modulus' ./ sig; 
          contour(time,log2(scales),sig,[-99 1],'k');
        end;
    end;
end


%-------------------------------------------------------------------------%


% plot areawise significance test
function plotat(time,scales,at,sigplot);
    if ((sigplot==2 | sigplot==3) & ~isnan(at))
            % check if this plot is desired and proper values given
        hold on
        contour(time,log2(scales),at',[-99 1],'k','LineWidth',5);
    end;
end

%-------------------------------------------------------------------------%


% plots cone-of-influence, anything "below" is dubious
function plotcoi(time,coi);
    hold on
    plot(time,log2(coi),'k')   
end


%-------------------------------------------------------------------------%


% marks scales and times by plotting dotted lines
function plotmarks(time,scales,marks,markt);
    if (markt(1) ~= -999) % check if vertical lines are desired
      hold on
      for (i = 1:length(markt))
        plot([markt(i);markt(i)],log2([min(scales);max(scales)]),':k','LineWidth',1)
      end;
    end;
    if (marks(1) ~= -999)% check if horizontal lines are desired
      hold on
      for (i = 1:length(marks))
        plot([min(time);max(time)],log2([marks(i),marks(i)]),':k','LineWidth',1)
      end;
    end;      
end


%-------------------------------------------------------------------------%


% function to calculate new clim values for plot areas to have different
% colormaps in single plot
% see Matlab Help 'Example -- Simulating Multiple Colormaps in a Figure'
function CLim = newclim(BeginSlot,EndSlot,CDmin,CDmax,CmLength)
    %                Convert slot number and range
    %                to percent of colormap
    PBeginSlot    = (BeginSlot - 1) / (CmLength - 1);
    PEndSlot      = (EndSlot - 1) / (CmLength - 1);
    PCmRange      = PEndSlot - PBeginSlot;
    %                Determine range and min and max 
    %                of new CLim values
    DataRange     = CDmax - CDmin;
    ClimRange     = DataRange / PCmRange;
    NewCmin       = CDmin - (PBeginSlot * ClimRange);
    NewCmax       = CDmax + (1 - PEndSlot) * ClimRange;
    CLim          = [NewCmin,NewCmax];
end
