%
% WCO - Wavelet Sample Coherency
% ------------------------------
%
% This funtion estimates the wavelet coherency of two time series objects
% with the Morlet wavelet.
%
%
% NEEDS: 
%   Wavelet software by C. Torrence and G. Compo
%     available at URL: http://paos.colorado.edu/research/wavelets/
%
%
% USAGE
% wclist =
%   wco(ts1,ts2,s0,noctave,nvoice,w0,sw,tw,swabs,siglevel,arealsiglevel,kernel,markt,marks,sqr,phase,plotvar,units,device,file,split,color,pwidth,pheight,labsc,labtext,sigplot)
%
%
% INPUT arguments and defaults: 
%
% mandatory input:
%   ts1:    first iddata time series object to be transformed
%   ts2:    second iddata time series object to be transformed
%
% optional input (has to appear in the right order, set -1 to obtain default):
%   s0(=1):          lowest calculated scale in units of the time series
%   noctave(=5):     number of octaves
%   nvoice(=10):     number of voices per octave
%   w0=(2*pi):       time/frequency resolution omega_0, parameter of Morlet Wavelet
%   sw(=0.5):        length of smoothing window in scale direction is 2*sw*nvoice+1
%   tw(=1.5):        length of smoothing window in time direction is 2*s*tw+1
%   swabs(=0)        length of smoothing window in scale direction at scale s is 2*swabs+1
%   siglevel(=0.95): vector of significance levels for pointwise test, e.g. [0.9 0.95], default 0.95.
%   arealsiglevel(=0.9): significance level of the areawise test; 
%                    currently only for siglevel=0.95 and for arealsiglevel=0.9 possible, 
%                    i.e. 90 percent of the area of false positive patches is sorted out
%   kernel(=0):      bitmap of the reproducing kernel; 
%                    if not provided, it will be calculated during the areawise test
%   markt(=-999):    vector of times to be marked by vertical dotted lines; when set to -999 (default), no lines are plotted.
%   marks(=-999):    vector of scales to be marked by horizontal dotted lines; when set to -999 (default), no lines are plotted.
%   sqr(=false):     if true, the squared coherency is given.
%   phase(=true):    true when phase calculation desired
%   plotvar(=true):  true when graphical output desired
%   units(=''):      character string giving units of the data sets.
%   split(=false):  when true, modulus and phase are splitted in two graphs.
%   color(=true):   true (default): color plot, false: gray scale
%   labsc(=1):      scale of labels, default: 1, for two-column manuscripts: 1.5, for presentations: >2
%   labtext(=''):   puts a label in upper left corner of the plot
%   sigplot(=3):    0: no significance test plotted, 1: results from pointwise test, 2: results from areawise test, 3: results from both tests
%
%
% OUTPUT arguments:
%
%   wclist:      contains the following parameters
%
%   wclist.modulus      here: wavelet sample spectrum of dimension [length(ts)]x[nvoice*noctave+1]
%   wclist.phase        not used
%   wclist.time         vector of times of length(intersection of ts1 and ts2)
%   wclist.s0           lowest calculated scale in units of the time series
%   wclist.noctave      number of octaves
%   wclist.nvoice       number of voices per octave
%   wclist.w0           time/frequency resolution omega_0, parameter of Morlet Wavelet
%   wclist.scales       vector of scales of length nvoice*noctave+1
%   wclist.critval      matrix of critical values
%   wclist.at           time/scale matrix of areawise significant patches
%   wclist.kernel       bitmap of the reproducing kernel
%   wclist.coi          cone of influence
%
%
% DETAILS
% The default of sw and tw is set to zero to avoid uncritical use of the
% function (the sample coherency makes sense only for sw or tw >0). A
% pointwise significance test is performed agaist an almost process
% independent background spectrum. If arealsiglevel=0.9, an areawise
% significance test is performed, that sorts out 90 percent of the area of
% false positive patches. The cone of influence is marked by black lines.
% Values outside the cone of influence should be interpreted very
% carefully, as they result from a significant contribution of zero padding
% at the beginning and the end of the time series. For a better
% visualization, additional dotted lines marking distinct times or scales
% might be plotted by providing the vectors markt and marks. The function
% returns an object of type "wt", that might be directly plotted by the
% plot function.
%
%
%--------------------------------------------------------------------------
%   Copyright (C) 2004-2007, Douglas Maraun and Jonas Laehnemann (Matlab adaption)
%
%   This software may be used, copied, or redistributed as long as it is not
%   sold and this copyright notice is reproduced on each copy made. This
%   routine is provided as is without any express or implied warranties
%   whatsoever.
%   
%   URL: http://tocsy.agnld.uni-potsdam.de/wavelets/
%   This package is also available as R routine.
%
% Notice: Please acknowledge the use of this software in any publications:
%    ``Wavelet coherency software was provided by D. Maraun and J. Laehnemann
%      and is available at URL: http://tocsy.agnld.uni-potsdam.de/wavelets/''.
%
% References:
%   D. Maraun, J. Kurths and M. Holschneider: Nonstationary Gaussian
%   Processes in Wavelet Domain: Synthesis, Estimation and Significance
%   Testing. Phys. Rev. E 75, 016707, 2007
%
%   D. Maraun and J. Kurths: Cross Wavelet Analysis. Significance Testing
%   and Pitfalls, Nonlin. Proc. Geoph. 11(4), 505-514, 2004
%
% If you have got any questions, please feel free to contact: 
%   Douglas Maraun, PhD
%   Climatic Research Unit, University of East Anglia
%   +44 1603 59 3857
%   d.maraun(at)uea.ac.uk
%   http://www.cru.uea.ac.uk/~douglas
%----------------------------------------------------------------------------

function [wclist] = wsp(ts1,ts2,s0,noctave,nvoice,w0,sw,tw,swabs,siglevel,arealsiglevel,kernel,markt,marks,sqr,phase,plotvar,units,split,color,labsc,labtext,sigplot);

  % CHECK INPUTS

  if (nargin < 23), sigplot = -1; end;
  if (nargin < 22), labtext = -1; end;
  if (nargin < 21), labsc = -1; end;
  if (nargin < 20), color = -1; end;
  if (nargin < 19), split = -1; end;
  if (nargin < 18), units = -1; end;
  if (nargin < 17), plotvar = -1; end;
  if (nargin < 16), phase = -1; end;
  if (nargin < 15), sqr = -1; end;
  if (nargin < 14), marks = -1; end;
  if (nargin < 13), markt = -1; end;
  if (nargin < 12), kernel = -1; end;
  if (nargin < 11), arealsiglevel = -1; end;
  if (nargin < 10), siglevel = -1; end;
  if (nargin < 9), swabs = -1; end;  
  if (nargin < 8), tw = -1; end;
  if (nargin < 7), sw = -1; end;
  if (nargin < 6), w0 = -1; end;
  if (nargin < 5), nvoice = -1; end;
  if (nargin < 4), noctave = -1; end;
  if (nargin < 3), s0 = -1; end;
  if (nargin < 2) 
      error('Must input two iddata TS object') 
  end

  % SET DEFAULTS - defaults for plotting variables are set in plotwt.m
  
  if (s0 == -1), s0=1; end;
  if (noctave == -1), noctave = 5; end;
  if (nvoice == -1), nvoice = 10; end;
  if (w0 == -1), w0 = 2*pi; end;
  if (sw == -1), sw = 0.5; end; % defaults changed
  if (tw == -1), tw = 1.5; end; % defaults changed
  if (swabs == -1), swabs = 0; end;
  if (siglevel == -1), siglevel = 0.95; end;
  if (arealsiglevel == -1), arealsiglevel = 0.9; end;
  if (kernel == -1), kernel = 0; end;
  if (sqr == -1), sqr = false; end;
  if (phase == -1), phase = true; end;  
  if (plotvar == -1), plotvar = true; end;
 
  % CHECK TIME SERIES INPUT 

  [ts1,ts2] = adjusttimeseries(ts1,ts2); % reduce timeseries to the time they both cover

  y1 = ts1(:,2)';
  y2 = ts2(:,2)';
  t = ts1(:,1)';
  dt= (ts1(length(y1),1)-ts1(1,1))/(length(y1)-1);

  % ADJUST AND CALCULATE NECESSARY PARAMETERS
  
  if (sw~=0 & swabs==0) % adjust smoothing windows in scale direction at scale s
    swabs = fix(sw*nvoice); end;
  if (swabs~=0 & sw==0)
    sw = swabs/nvoice; end;
  
  [siglevel,arealsiglevel] = checkarealsiglevel(sw,tw,w0,arealsiglevel,siglevel,1);
  
  if (sw==0 & tw==0 & swabs==0)
    Message_wco = '# coherence without averaging makes no sense!'
    siglevel = 0;
    arealsiglevel = 0;
  end;
    
  at = NaN; % set areal test result to NaN, in case areal test is not performed
  
  s0rem = s0;
  s0 = adjusts0(s0,dt);
  dnoctave = fix(log(s0/s0rem-0.000000000001)/log(2))+1;
  
  noctave = adjustnoctave(length(y1),dt,s0,tw,noctave);

  y1anom = y1-mean(y1);  % substract mean from timeseries
  y2anom = y2-mean(y2);
  
  dj=1/nvoice; % the spacing between discrete scales
  J1=nvoice*noctave; % the # of scales minus one

  % PERFORM WAVELET TRANSFORMS OF TIMESERIES
  [ts1cwt,exactscales,scalevector,coi] = wavelet(y1anom,dt,1,dj,s0,J1,'Morlet',w0);
      % zeropadding is activated  
  ts1cwt = ts1cwt.'; % transpose to fit in with the other routines
  ts2cwt = wavelet(y2anom,dt,1,dj,s0,J1,'Morlet',w0); % scalevector and coi are the same and need not be calculated again
  ts2cwt = ts2cwt.'; % transpose to fit in with the other routines

  % CALCULATE WCO and SMOOTHING
  cosp = real(ts1cwt).*real(ts2cwt) + imag(ts1cwt).*imag(ts2cwt);
  quad = imag(ts1cwt).*real(ts2cwt) - real(ts1cwt).*imag(ts2cwt);
  wsp1 = abs(ts1cwt).^2;
  wsp2 = abs(ts2cwt).^2;
  
  smcosp = smoothmatrix(cosp,swabs);
  smquad = smoothmatrix(quad,swabs);
  smwsp1 = smoothmatrix(wsp1,swabs);
  smwsp2 = smoothmatrix(wsp2,swabs);
  
  smsmcosp = smoothtime(smcosp,tw,dt,scalevector);
  smsmquad = smoothtime(smquad,tw,dt,scalevector);
  smsmwsp1 = smoothtime(smwsp1,tw,dt,scalevector);
  smsmwsp2 = smoothtime(smwsp2,tw,dt,scalevector);

  if (sqr==false) % give squared coherency or not 
    wcoh = sqrt((smsmcosp.^2+smsmquad.^2)./(smsmwsp1.*smsmwsp2));
  else
    wcoh = (smsmcosp.^2+smsmquad.^2)./(smsmwsp1.*smsmwsp2);
  end;
  
  % CALUCLATE PHASE, if desired
  if (phase)
    phs = atan2(smsmquad,smsmcosp);
  else 
    phs = NaN;
  end;
  
  % POINTWISE SIGNIFICANCE TEST
  if (siglevel(1)~=0) critval = criticalvaluesWCO(s0,noctave,nvoice,w0,swabs,tw,siglevel);
  else critval = NaN; end;
  if (sqr==true & isnan(critval)==false)
    critval = critval.^2;
  end;

  % AREAWISE SIGNIFICANCE TEST
  if (arealsiglevel~=0)
    [at,kernel] = arealtest(wcoh./critval,dt,s0,noctave,nvoice,w0,swabs,tw,siglevel,arealsiglevel,kernel,1);
  end;

  wcoh(1,1) = 0; % values to norm color scale 
  wcoh(1,2) = 1;

  if (phase)
    phs(1,1) = -pi; % values to norm color scale in phase-plot
    phs(1,2) = pi;
  end;

  if (s0rem<s0)
    wcoh = addvalues(nvoice,dnoctave,wcoh,NaN);
    phs = addvalues(nvoice,dnoctave,phs,NaN);
    noctave = noctave+dnoctave;
    s0 = s0/2^dnoctave;
    scalevector = 2.^((0:(noctave*nvoice))./nvoice).*s0;
  end;
    
  % PARAMETERS - producing wt structure, details see above
  wclist.modulus = wcoh; 
  wclist.phase = phs; 
  wclist.time = t; 
  wclist.s0 = s0; 
  wclist.noctave = noctave; 
  wclist.nvoice = nvoice;
  wclist.w0 = w0; 
  wclist.scales = scalevector; 
  wclist.critval = critval; 
  wclist.at = at;
  wclist.kernel = kernel; 
  wclist.coi = coi; 
  
  % PLOTTING
  if (plotvar) 
      plotwt(wclist,markt,marks,false,phase,units,'Wavelet Sample Coherency',split,color,labsc,labtext,sigplot);
  end;
 
end
