%
% WCS - Wavelet Sample Cross Spectrum
% -----------------------------------
%
% This funtion estimates the wavelet cross spectrum of two time series
% objects with the Morlet wavelet.
%
%
% NEEDS: 
%   Wavelet software by C. Torrence and G. Compo
%     available at URL: http://paos.colorado.edu/research/wavelets/
%
%
% USAGE
% wclist =
%   (ts1,ts2,s0,noctave,nvoice,w0,sw,tw,swabs,markt,marks,logscale,phase,plotvar,units,split,color,labsc,labtext)
%
%
% INPUT arguments and defaults: 
%
% mandatory input:
%   ts1:    first time series to be transformed (two column matrix, 1st col: times, 2nd col: values); 
%           if only a vector is given, dt=1 and t0=1 are assumed
%           
%   ts2:    second time series to be transformed (two column matrix, 1st col: times, 2nd col: values);
%           if only a vector is given, dt=1 and t0=1 are assumed
%
% optional input (has to appear in the right order, set -1 to obtain default):
%   s0(=1):          lowest calculated scale in units of the time series
%   noctave(=5):     number of octaves
%   nvoice(=10):     number of voices per octave
%   w0=(2*pi):       time/frequency resolution omega_0, parameter of Morlet Wavelet
%   sw(=0):          length of smoothing window in scale direction is 2*sw*nvoice+1
%   tw(=0):          length of smoothing window in time direction is 2*s*tw+1
%   swabs(=0)        length of smoothing window in scale direction at scale s is 2*swabs+1
%   markt(=-999):    vector of times to be marked by vertical dotted lines; when set to -999 (default), no lines are plotted.
%   marks(=-999):    vector of scales to be marked by horizontal dotted lines; when set to -999 (default), no lines are plotted.
%   logscale(=false):when true, the contours are plotted in logarithmic scale
%   phase(=true):    true when phase calculation desired
%   plotvar(=true):  true when graphical output desired
%   units(=''):      character string giving units of the data sets.
%   split(=false):   when true, modulus and phase are splitted in two graphs.
%   color(=true):    true (default): color plot, false: gray scale
%   labsc(=1):       scale of labels, default: 1, for two-column manuscripts: 1.5, for presentations: >2
%   labtext(=''):    puts a label in upper left corner of the plot
%
%
% OUTPUT arguments:
%
%   wclist:      contains the following parameters
%
%   wclist.modulus      here: wavelet sample spectrum of dimension [length(ts)]x[nvoice*noctave+1]
%   wclist.phase        not used
%   wclist.time         vector of times of length(intersection of ts1 and ts2)
%   wclist.s0           lowest calculated scale in units of the time series
%   wclist.noctave      number of octaves
%   wclist.nvoice       number of voices per octave
%   wclist.w0           time/frequency resolution omega_0, parameter of Morlet Wavelet
%   wclist.scales       vector of scales of length nvoice*noctave+1
%   wclist.critval      matrix of critical values
%   wclist.at           time/scale matrix of areawise significant patches
%   wclist.kernel       bitmap of the reproducing kernel
%   wclist.coi          cone of influence
%
%
% DETAILS
% WARNING! Better do not use this function because it is in general easily
% misinterpreted! A peak in the wavelet cross sample spectrum appears in the
% three cases, that either of the processes exhibits a peak (what is uninteresting), 
% or both processes covary (what is interesting). But it does not tell, 
% what case is observed. So in general, a peak in the wavelet cross sample spectrum 
% does not imply that the two underlying processes are related in any way. 
% The function returns an object of type "wt", that might be directly plotted by the plot
% function.
%
%
%--------------------------------------------------------------------------
%   Copyright (C) 2004-2007, Douglas Maraun and Jonas Laehnemann (Matlab adaption)
%
%   This software may be used, copied, or redistributed as long as it is not
%   sold and this copyright notice is reproduced on each copy made. This
%   routine is provided as is without any express or implied warranties
%   whatsoever.
%   
%   URL: http://tocsy.agnld.uni-potsdam.de/wavelets/
%   This package is also available as R routine.
%
% Notice: Please acknowledge the use of this software in any publications:
%    ``Wavelet coherency software was provided by D. Maraun and J. Laehnemann
%      and is available at URL: http://tocsy.agnld.uni-potsdam.de/wavelets/''.
%
% or cite these two relevant references:
%   D. Maraun, J. Kurths and M. Holschneider: Nonstationary Gaussian
%   Processes in Wavelet Domain: Synthesis, Estimation and Significance
%   Testing. Phys. Rev. E 75, 016707, 2007
%
%   D. Maraun and J. Kurths: Cross Wavelet Analysis. Significance Testing
%   and Pitfalls, Nonlin. Proc. Geoph. 11(4), 505-514, 2004
%
% If you have got any questions, please feel free to contact: 
%   Douglas Maraun, PhD
%   Climatic Research Unit, University of East Anglia
%   +44 1603 59 3857
%   d.maraun(at)uea.ac.uk
%   http://www.cru.uea.ac.uk/~douglas
%--------------------------------------------------------------------------



function [wclist] = wcs(ts1,ts2,s0,noctave,nvoice,w0,sw,tw,swabs,markt,marks,logscale,phase,plotvar,units,split,color,labsc,labtext);

  % CHECK INPUTS

  if (nargin < 19), labtext = -1; end;
  if (nargin < 18), labsc = -1; end;
  if (nargin < 17), color = -1; end;
  if (nargin < 16), split = -1; end;
  if (nargin < 15), units = -1; end;
  if (nargin < 14), plotvar = -1; end;
  if (nargin < 13), phase = -1; end;
  if (nargin < 12), logscale = -1; end;
  if (nargin < 11), marks = -1; end;
  if (nargin < 10), markt = -1; end;
  if (nargin < 9), swabs = -1; end;  
  if (nargin < 8), tw = -1; end;
  if (nargin < 7), sw = -1; end;
  if (nargin < 6), w0 = -1; end;
  if (nargin < 5), nvoice = -1; end;
  if (nargin < 4), noctave = -1; end;
  if (nargin < 3), s0 = -1; end;
  if (nargin < 2) 
      error('Must input two time series') 
  end

  % SET DEFAULTS - defaults for plotting variables are set in plotwt.m
  
  if (s0 == -1), s0=1; end;
  if (noctave == -1), noctave = 5; end;
  if (nvoice == -1), nvoice = 10; end;
  if (w0 == -1), w0 = 2*pi; end;
  if (sw == -1), sw = 0; end;
  if (tw == -1), tw = 0; end;
  if (swabs == -1), swabs = 0; end;
  if (phase == -1), phase = true; end;  
  if (plotvar == -1), plotvar = true; end;

  % CHECK TIME SERIES INPUT 

  [ts1,ts2] = adjusttimeseries(ts1,ts2); % reduce timeseries to the time they both cover

  y1 = ts1(:,2)';
  y2 = ts2(:,2)';
  t = ts1(:,1)';
  dt= (ts1(length(y1),1)-ts1(1,1))/(length(y1)-1);

  %  ADJUST AND CALCULATE NECESSARY PARAMETERS

  if (sw~=0 & swabs==0) % adjust smoothing windows in scale direction at scale s
    swabs = fix(sw*nvoice); end;

  s0rem = s0;
  s0 = adjusts0(s0,dt);
  dnoctave = fix(log(s0/s0rem-0.000000000001)/log(2))+1;
  
  noctave = adjustnoctave(length(y1),dt,s0,tw,noctave);  
  
  y1anom = y1-mean(y1); % substract mean from timeseries
  y2anom = y2-mean(y2);
  
  dj=1/nvoice; % the spacing between discrete scales
  J1=nvoice*noctave; % the # of scales minus one

  % PERFORM WAVELET TRANSFORMS OF TIMESERIES
  [ts1cwt,exactscales,scalevector,coi] = wavelet(y1anom,dt,1,dj,s0,J1,'Morlet',w0);
          % zeropadding is activated
  ts1cwt = ts1cwt.'; % transpose to fit in with the other routines
  ts2cwt = wavelet(y2anom,dt,1,dj,s0,J1,'Morlet',w0); % scalevector and coi are the same and need not be calculated again
  ts2cwt = ts2cwt.'; % transpose
  
  % CALCULATE WCS and SMOOTHING
  wcsp = ts1cwt .* ts2cwt;

  cosp = real(ts1cwt).*real(ts2cwt) + imag(ts1cwt).*imag(ts2cwt);
  quad = imag(ts1cwt).*real(ts2cwt) - real(ts1cwt).*imag(ts2cwt);
  
  smcosp = smoothmatrix(cosp,swabs);
  smquad = smoothmatrix(quad,swabs);
  
  smsmcosp = smoothtime(smcosp,tw,dt,scalevector);
  smsmquad = smoothtime(smquad,tw,dt,scalevector);
  
  wcsp = smsmcosp.^2 + smsmquad.^2;

  % CALUCLATE PHASE, if desired
  if (phase)
    phs = atan2(smsmquad,smsmcosp);
  else 
    phs = NaN; 
  end;
  
  if (s0rem<s0)
    wcsp = addvalues(nvoice,dnoctave,wcsp,NaN);
    phs = addvalues(nvoice,dnoctave,phs,NaN);
    noctave = noctave+dnoctave;
    s0 = s0/2^dnoctave;
    scalevector = 2.^((0:(noctave*nvoice))./nvoice).*s0;
  end;
  
  % PARAMETERS - producing wt structure, details see above
  wclist.modulus = wcsp;
  wclist.phase = phs;
  wclist.time = t;
  wclist.s0 = s0;
  wclist.noctave = noctave;
  wclist.nvoice = nvoice;
  wclist.w0 = w0;
  wclist.scales = scalevector;
  wclist.critval = NaN; % Significance tests were not performed
  wclist.at = NaN;
  wclist.kernel = NaN;
  wclist.coi = coi;
   
  % PLOTTING
  if (plotvar)
      plotwt(wclist,markt,marks,logscale,phase,units,'Wavelet Cross Spectrum',split,color,labsc,labtext,1);
  end;
 
end
