%
% WSP - Wavelet Sample Spectrum
% -----------------------------
% This funtion estimates (and possibly plots) the wavelet spectrum of a 
% time series object with the Morlet wavelet.
%
%
% NEEDS: 
%   - Matlab 'System Identification Toolbox' for ar and sim in criticalvaluesWSP
%   - Wavelet software by C. Torrence and G. Compo
%     available at URL: http://paos.colorado.edu/research/wavelets/
%
%
% USAGE
% wclist =
%   wsp(ts,s0,noctave,nvoice,w0,sw,tw,swabs,siglevel,critval,nreal,
%   arealsiglevel,kernel,markt,marks,logscale,plot,units,device,file,
%   color,pwidth,pheight,labsc,labtext,sigplot) 
%
%
% INPUT arguments and defaults: 
%
% mandatory input:
%   ts:              iddata timeseries object to be transformed
%
% optional input (has to appear in the right order, set -1 to obtain default):
%   s0(=1):          lowest calculated scale in units of the time series
%   noctave(=5):     number of octaves
%   nvoice(=10):     number of voices per octave
%   w0=(2*pi):       time/frequency resolution omega_0, parameter of Morlet Wavelet
%   sw(=0):          length of smoothing window in scale direction is 2*sw*nvoice+1
%   tw(=0):          length of smoothing window in time direction is 2*s*tw+1
%   swabs(=0)        length of smoothing window in scale direction at scale s is 2*swabs+1
%   siglevel(=0.95): vector of significance levels for pointwise test, e.g. [0.9 0.95], default 0.95.
%   critval(=NULL):  matrix of critical values calculated e.g. by criticalvaluesWSP; 
%                    each row contains critical values for the corresponding chosen significance level.
%   nreal(=1000):    number of realizations to estimate critical values for the corresponding significance values
%   arealsiglevel(=0.9): significance level of the areawise test; 
%                    currently only for siglevel=0.95 and for arealsiglevel=0.9 possible, 
%                    i.e. 90 percent of the area of false positive patches is sorted out
%   kernel(=0):      bitmap of the reproducing kernel; 
%                    if not provided, it will be calculated during the areawise test
%   markt(=-999):    vector of times to be marked by vertical dotted lines; when set to -999 (default), no lines are plotted.
%   marks(=-999):    vector of scales to be marked by horizontal dotted lines; when set to -999 (default), no lines are plotted.
%   logscale(=false): when true, the contours are plotted in logarithmic scale
%   plotvar(=true):  true when graphical output desired
%   units(=''):      character string giving units of the data sets.
%   color(=true):   true (default): color plot, false: gray scale
%   labsc(=1):      scale of labels, default: 1, for two-column manuscripts: 1.5, for presentations: >2
%   labtext(=''):   puts a label in upper left corner of the plot
%   sigplot(=3):    0: no significance test plotted, 1: results from pointwise test, 2: results from areawise test, 3: results from both tests
%
%
% OUTPUT arguments:
%
%   wclist:      contains the following parameters
%
%   wclist.modulus      here: wavelet sample spectrum of dimension [length(ts)]x[nvoice*noctave+1]
%   wclist.phase        not used
%   wclist.time         vector of times of length(intersection of ts1 and ts2)
%   wclist.s0           lowest calculated scale in units of the time series
%   wclist.noctave      number of octaves
%   wclist.nvoice       number of voices per octave
%   wclist.w0           time/frequency resolution omega_0, parameter of Morlet Wavelet
%   wclist.scales       vector of scales of length nvoice*noctave+1
%   wclist.critval      matrix of critical values
%   wclist.at           time/scale matrix of areawise significant patches
%   wclist.kernel       bitmap of the reproducing kernel
%   wclist.coi          cone of influence
%
%
% DETAILS
% A pointwise significance test is performed either by providing critical 
% values (e.g. from critical.valuesWSP) or by Monte Carlo simulations. If 
% critval and either nreal or siglevel is zero, no significance test is 
% performed. If arealsiglevel=0.9, an areawise significance test is
% performed, that sorts out 90 percent of the area of false positive
% patches. The cone of influence is marked by black lines. Values outside
% the cone of influence should be interpreted very carefully, as they
% result from a significant contribution of zero padding at the beginning
% and the end of the time series. For a better visualization, additional
% dotted lines marking distinct times or scales might be plotted by
% providing the vectors markt and marks. In the linear plot, values are
% normalized such that the highest spetral power equals one. In the
% logarithmic plot, values are normalized, that the lowest value equals
% 10^0=1. The function returns an object of type "wt", that might be
% directly plotted by the plot function.
%
%
%--------------------------------------------------------------------------
%   Copyright (C) 2004-2007, Douglas Maraun and Jonas Laehnemann (Matlab adaption)
%
%   This software may be used, copied, or redistributed as long as it is not
%   sold and this copyright notice is reproduced on each copy made. This
%   routine is provided as is without any express or implied warranties
%   whatsoever.
%   
%   URL: http://tocsy.agnld.uni-potsdam.de/wavelets/
%   This package is also available as R routine.
%
% Notice: Please acknowledge the use of this software in any publications:
%    ``Wavelet coherency software was provided by D. Maraun and J. Laehnemann
%      and is available at URL: http://tocsy.agnld.uni-potsdam.de/wavelets/''.
%
% References:
%   D. Maraun, J. Kurths and M. Holschneider: Nonstationary Gaussian
%   Processes in Wavelet Domain: Synthesis, Estimation and Significance
%   Testing. Phys. Rev. E 75, 016707, 2007
%
%   D. Maraun and J. Kurths: Cross Wavelet Analysis. Significance Testing
%   and Pitfalls, Nonlin. Proc. Geoph. 11(4), 505-514, 2004
%
% If you have got any questions, please feel free to contact: 
%   Douglas Maraun, PhD
%   Climatic Research Unit, University of East Anglia
%   +44 1603 59 3857
%   d.maraun(at)uea.ac.uk
%   http://www.cru.uea.ac.uk/~douglas
%--------------------------------------------------------------------------


function [wclist] = wsp(ts,s0,noctave,nvoice,w0,sw,tw,swabs,siglevel,critval,nreal,arealsiglevel,kernel,markt,marks,logscale,plotvar,units,color,labsc,labtext,sigplot)

  % CHECK INPUTS

  if (nargin < 22), sigplot = -1; end;
  if (nargin < 21), labtext = -1; end;
  if (nargin < 20), labsc = -1; end;
  if (nargin < 19), color = -1; end;
  if (nargin < 18), units = -1; end;
  if (nargin < 17), plotvar = -1; end;
  if (nargin < 16), logscale = -1; end;
  if (nargin < 15), marks = -1; end;
  if (nargin < 14), markt = -1; end;
  if (nargin < 13), kernel = -1; end;
  if (nargin < 12), arealsiglevel = -1; end;
  if (nargin < 11), nreal = -1; end;
  if (nargin < 10), critval = -1; end;
  if (nargin < 9), siglevel = -1; end;
  if (nargin < 8), swabs = -1; end;  
  if (nargin < 7), tw = -1; end;
  if (nargin < 6), sw = -1; end;
  if (nargin < 5), w0 = -1; end;
  if (nargin < 4), nvoice = -1; end;
  if (nargin < 3), noctave = -1; end;
  if (nargin < 2), s0 = -1; end;
  if (nargin < 1) 
      error('Must input an iddata TS object') 
  end

  % SET DEFAULTS - defaults for plotting variables are set in plotwt.m
  
  if (s0 == -1), s0=1; end;
  if (noctave == -1), noctave = 5; end;
  if (nvoice == -1), nvoice = 10; end;
  if (w0 == -1), w0 = 2*pi; end;
  if (sw == -1), sw = 0; end;
  if (tw == -1), tw = 0; end;
  if (swabs == -1), swabs = 0; end;
  if (siglevel == -1), siglevel = 0.95; end;
  if (critval == -1), critval = NaN; end;
  if (nreal == -1), nreal = 1000; end;
  if (arealsiglevel == -1), arealsiglevel = 0.9; end;
  if (kernel == -1), kernel = 0; end;
  if (plotvar == -1), plotvar = true; end;
   
  % CHECK TIME SERIES INPUT 

  if (size(ts,2)==2)
    y = ts(:,2)';
    t = ts(:,1)';
    dt= (ts(length(y),1)-ts(1,1))/(length(y)-1);
  end;  
  if (size(ts,2)==1)
    y = ts;
    dt = (ts(length(y),1)-ts(1,1))/(length(y)-1);
    t = (1:dt:length(y));
  end;
  if (size(ts,2)>2)
    error('please provide two column matrix for time series');end;
  
  % ADJUST AND CALCULATE NECESSARY PARAMETERS
  
  if (sw~=0 & swabs==0)
    swabs = fix(sw*nvoice); end;
  if (swabs~=0 & sw==0)
    sw = swabs/nvoice; end;
  
  [siglevel,arealsiglevel] = checkarealsiglevel(sw,tw,w0,arealsiglevel,siglevel,0);

  at = NaN;

  s0rem = s0;
  s0 = adjusts0(s0,dt);
  dnoctave = fix(log(s0/s0rem-0.000000000001)/log(2))+1;

  noctave = adjustnoctave(length(y),dt,s0,tw,noctave);
  
  yanom = y-mean(y);  % substract mean from timeseries
  tsanom = [t' yanom'];  

  dj=1/nvoice; % the spacing between discrete scales
  J1=nvoice*noctave; % the # of scales minus one

  % WAVELET TRANSFORMATION
  [tscwt,phase,scalevector,coi] = wavelet(yanom,dt,1,dj,s0,J1,'Morlet',w0); 
        % zeropadding activated
  tscwt = tscwt.'; % transpose to fit in with the other routines

  % SMOOTHING
  wsp = smoothmatrix(abs(tscwt).^2,swabs);
  smwsp = smoothtime(wsp,tw,dt,scalevector);

  % POINTWISE SIGNIFICANCE TEST
  if (isnan(critval)==false) % is critval empty?
    if (size(critval,2)~=size(smwsp,2)) % is critval of the wrong dimension?
      if (siglevel(1)~=0 & nreal~=0) 
          critval = criticalvaluesWSP(tsanom,s0,noctave,nvoice,w0,swabs,tw,siglevel,nreal);
          % critval is of wrong dimension and siglevel and nreal are given
      else
        critval = NaN; % no test possible
        arealsiglevel = 0;
        Message_wsp = strvcat('# dimension of critval is wrong','# areawise test only possible with pointwise test')
      end;
    end;
  else % critval is empty, but nreal or siglevel is given
    if (siglevel(1)~=0 & nreal~=0) 
        critval = criticalvaluesWSP(tsanom,s0,noctave,nvoice,w0,swabs,tw,siglevel,nreal);
    else 
      critval = NaN;
      arealsiglevel = 0;
      Message_wsp = ('# areawise test only possible with pointwise test')
    end;
  end;
  
  % AREAL SIGNIFICANCE TEST
  if (arealsiglevel~=0)
    v = critval(1,:);
    v(v==0) = 9999;
    cvmat = repmat(v,length(t),1);
    [at,kernel] = arealtest(smwsp./cvmat,dt,s0,noctave,nvoice,w0,swabs,tw,siglevel,arealsiglevel,kernel,0);
  end;

  if (s0rem<s0)
    smwsp = addvalues(nvoice,dnoctave,smwsp,NaN);
    critval = addvalues(nvoice,dnoctave,critval,1);
    %at = addvalues(nvoice,dnoctave,at,NA)
    noctave = noctave+dnoctave;
    s0 = s0/2^dnoctave;
    scalevector = 2.^((0:(noctave*nvoice))./nvoice).*s0;
  end;

  % PARAMETERS - producing wt structure, details see above
  wclist.modulus = smwsp;
  wclist.phase = NaN;
  wclist.time = t;
  wclist.s0 = s0;
  wclist.noctave = noctave;
  wclist.nvoice = nvoice;
  wclist.w0 = w0;
  wclist.scales = scalevector;
  wclist.critval = critval;
  wclist.at = at;
  wclist.kernel = kernel;
  wclist.coi = coi;

  % PLOTTING
  if (plotvar)
      plotwt(wclist,markt,marks,logscale,false,units,'Wavelet Power Spectrum',false,color,labsc,labtext,sigplot);
  end;

 
end
